<?php
/**
 * Clips setups, checks and admin menu.
 * 
 * @package WP_Clips/Includes
 * @subpackage Admin
 */


class WP_Clips_Admin {


	public function __construct() {
		
		// Check for multisite
		if( is_multisite() ) {
			add_action( 'admin_print_footer_scripts', function() {
				printf( '<div class="notice notice-error"><p><b>%s</b> %s <a href="https://wpclips.net/wp-clips-multisite/" target="_blank">%s</a>.</p></div>',
					__( 'WP Clips', 'wp-clips' ),
					__( 'is not compatible with multisite. Delete the plugin and install', 'wp-clips' ),
					__( 'WP Clips Multisite', 'wp-clips' )
				);
			});
			return;
		}

		// Build Clips directory
		if( ! file_exists( WP_CLIPS_PREC ) ) $this->clips_builder();

		// Actions on upload and rebuild submits
		if( isset( $_POST[ 'clips-upload' ] ) ) $this->clips_upload();	
		if( isset( $_POST[ 'clips-rebuild' ] ) ) $this->rebuild();

		add_action( 'after_setup_theme', [ $this, 'clips_check' ], 4 );
		add_action( 'admin_menu', [ $this, 'clips_menu' ] );
		add_action( 'wp_clips_manager', [ $this, 'clips_control' ] );
	}

	/**
	 * Add menu to WordPress admin
	 * @since 3.0
	 */
	public function clips_menu() {

		add_options_page(
				__( 'WP Clips Manager', 'wp-clips' ),
				__( 'Clips', 'wp-clips' ),
				'manage_options',
				'wp-clips-control',
				[ $this, 'clips_manager' ]
			);
	}

	/**
	 * Install or update Precoded Clips
	 * @since 3.0
	 */
	public function clips_upload() {

		if( ! $_FILES[ 'zip_file' ][ 'name' ] ) return;

		$filename = $_FILES[ 'zip_file' ][ 'name' ];
		$source = $_FILES[ 'zip_file' ][ 'tmp_name' ];
		$type = $_FILES[ 'zip_file' ][ 'type' ];
		$accepted_types = [ 'application/zip', 'application/x-zip-compressed', 'multipart/x-zip', 'application/x-compressed' ];
	
		// Destination before unzip
		$target = WP_CLIPS_ROOT . $filename;
		
		// Get current Clips
		$clips = $this->clips([ WP_CLIPS_PREC, WP_CLIPS_UNCL ]);

		// Check for compatible file
		$zip_check = in_array( $type, $accepted_types ) && substr( $filename, -4 ) == '.zip';
		$clip_check = substr( $filename, 0, 5 ) == 'clip-';

		// Isolate name and version
		$filebase = rtrim( $filename, '.zip' );
		$filedata = explode( '.', $filebase, 2 );
		$name = $filedata[0];
		$vers = isset( $filedata[1] ) ? $filedata[1] : false;

		$class = 'notice-error'; // Default

		// Get version of older file (if exists)
		if( $clip_check ) {
			foreach( $clips as $slug => $data ) {
				if( ! empty( $data['clipVers'] ) && $slug === $name )
					$current_vers = $data['clipVers'];
			}
		}

		// Check if file meets name and type protocols
		if( ! $zip_check || ! $clip_check || ! $vers )
			$notice = __( 'The file is not a Clip version or zip file. Please try again.', 'wp-clips' );

		// Check is newer version
		elseif( isset( $current_vers ) && version_compare( $vers, $current_vers, '<' ) )
			$notice = __( 'The source file is an older version and will not be installed', 'wp-clips' );

		// Check for PHP ZIP module
		elseif( ! class_exists( 'ZipArchive' ) )
			$notice = __( 'The server\'s PHP ZIP module must be enabled for uploads. Please contact your host.', 'wp-clips' );

		// Upload and extract file if checks
		elseif( move_uploaded_file( $source, $target ) ) {

			$zip = new ZipArchive();
			$x = $zip->open( $target );
			if( $x === true ) {

				$msg = __( 'was uploaded and installed', 'wp-clips' );
				$class = 'notice-success';
				$path = WP_CLIPS_PREC;

				if( $clip_check ) {

					// Cleanup
					foreach( $clips as $slug => $data ) 
						if( $slug === $name ) {
							$path = dirname( $data['clipPath'] );
							$this->delete( $data['clipPath'] );
							$msg = __( 'was updated', 'wp-clips' );
						}
				}

				$notice = sprintf( '<b>%s</b> %s.', $name, $msg );
				$zip->extractTo( $path );
				$zip->close();
				if( file_exists( $target ) ) unlink( $target );
			}
		}

		// Error if unsuccessful
		else $notice = __( 'There was a problem with the upload. Please try again.', 'wp-clips' );

		// Display notice
		add_action( 'admin_print_footer_scripts', function() use( $class, $notice ) {
			printf( '<div class="notice %s"><p>%s</p></div>', $class, $notice );
		});
	}

	/**
	 * Render Manager page
	 * @since 3.1.1
	 */
	public function clips_manager() {

		?>
        <div class="wrap">
			<h1><?php _e( 'WP Clips Manager', 'wp-clips' ); ?></h1>
            <p><?php
            	echo __( 'Plugin Version ', 'wp-clips' ) . WP_CLIPS_VERS;

				// Append update link (if new version exists)
				$xml_file = wp_remote_retrieve_body( wp_remote_get( 'https://wpclips.net/versions/wp-clips.xml' ) );
				$xml = simplexml_load_string( $xml_file );
				if( version_compare( $xml->vers, WP_CLIPS_VERS, '>' ) )
					printf( ' &#10141; <a href="https://wpclips.net/updating-wp-clips/">%s</a>', __( 'Update Available', 'wp-clips' ) );
			?></p><hr>
			<h3><?php _e( 'Install / Update Clips', 'wp-clips' ); ?></h3>

			<form enctype="multipart/form-data" method="post">
				<label><?php _e( 'Select a compatible Clip file (.zip):', 'wp-clips' ); ?></label>
				<input type="file" name="zip_file" />
				<?php submit_button( 'Install / Update', '', 'clips-upload', true ); ?>
			</form>

            <?php do_action( 'wp_clips_manager' ); ?>
		</div>
	<?php
	}

	/**
	 * Add Clips Control
	 * @since 3.1
	 */
	public function clips_control() {

		?>
		<hr />
        <h3><?php _e( 'Control', 'wp-clips' ); ?></h3>
		<p><?php _e( 'Clips can be activated or unclipped. Precoded Clips can also be deleted after being unclipped.', 'wp-clips' ); ?></p>

		<form method="post">
			<ul>
				<p><span class="dashicons dashicons-yes"></span>
				<span class="dashicons dashicons-minus"></span>
				<span class="dashicons dashicons-no-alt"></span>
				</p><?php

				// Get and sort Clips
				$clips = $this->clips();
				ksort( $clips );
				$clips = [ 'clip-core' => $clips['clip-core'], 'clip-custom' => $clips['clip-custom'] ] + $clips;

				foreach( $clips as $slug => $data ) {

					extract( $data );
					$html = '<li>';

					foreach( [ 'active', 'unclipped', 'delete' ] as $btn ) {

						$html .= sprintf( '<input type="radio" name="%s" value="%s"', $slug, $btn );
						switch( $btn ) {
							case 'active':
								$html .= $clipActv ? 'checked' : '';
								$text = __( 'Active', 'wp-clips' );
								break;
							case 'unclipped':
								$html .= $clipActv ? '' : 'checked';
								$text = __( 'Unclipped', 'wp-clips' );
								break;
							case 'delete':
								$text = __( 'Delete', 'wp-clips' );
								$html .= empty( $clipVers ) || $clipActv || isset( $clipSafe ) ? ' disabled' : '';
								$alert = __( 'Do you really want to delete this Clip', 'wp-clips' );
								$html .= sprintf( ' onclick="return confirm(%s%s%s)"', "'", $alert, "?'" );
						}
						$html .= sprintf( '> </input><span class="screen-reader-text">%s</span>', $text );
					}

					$html .= $clipName;
					$html .= empty( $clipVers ) ? '' : sprintf( ' (v%s)', $clipVers );

					if( ! empty( $authName ) ) {
						$html .= ! empty( $authPage ) ?
							sprintf( ' by <a href="%s" target="_blank">%s</a>', esc_url( $authPage ), $authName )
						:
							sprintf( ' by %s', $authName );
					}

					if( ! empty( $clipPage ) ) {
						$html .= sprintf( ' &#10141; <a href="%s" target="_blank">%s</a>', esc_url( $clipPage ), __( 'Guide', 'wp-clips' ) );
					}

					unset( $clipVers, $clipPage, $clipSafe, $authName, $authPage );
					$html = apply_filters( 'wp_clips_manager_clip_meta', $html, $slug, $data );

					echo $html . '</li>';
				}?>
			</ul>
			<?php submit_button( 'Rebuild Clips', 'primary', 'clips-rebuild', true ); ?>
		</form>
	<?php
	}

	/**
	 * Build Clips directories
	 * @since 3.0
	 */
	private function clips_builder() {

		// Build directories (if do not exist)
		if( ! file_exists( WP_CLIPS_PREC ) )
			mkdir( WP_CLIPS_PREC, 0755, true);

		if( ! file_exists( WP_CLIPS_UNCL ) )
			mkdir( WP_CLIPS_UNCL, 0755, true);

		// Build Clips from template (if do not exist)
		foreach( glob( WP_CLIPS_ROOT . 'templates/*' ) as $clip ) {

			$clipname = substr( basename( $clip ), 0, -5 );

			if( ! file_exists( WP_CLIPS_DIR . $clipname ) &&
				! file_exists( WP_CLIPS_UNCL . $clipname )
			) {
				mkdir( WP_CLIPS_DIR . $clipname, 0755, true);
				foreach( glob( $clip . '/*' ) as $file )
					copy( $file, WP_CLIPS_DIR . $clipname . '/' . basename( $file ) );
			}
		}
	}
	
	/**
	 * Check Clip compatibility
	 * @since 3.0
	 */
	public function clips_check() {

		$actv_theme = [ get_template(), is_child_theme() ? get_stylesheet() : '' ];
		$actv_plugins = [];

		foreach( get_option( 'active_plugins' ) as $plugin ) {
			list( $plugin_name ) = explode( '/', $plugin );
			$actv_plugins[] = $plugin_name;
		}

		foreach( $this->clips([ WP_CLIPS_DIR, WP_CLIPS_PREC ]) as $slug => $data ) {
			
			if( ! $data['clipActv'] ) continue;

			$notices = [];

			// Check required plugins are active
			if( ! empty( $data['depsPlug'] ) ) {
				$plugins_deps = explode( ',', $data['depsPlug'] );
				$plugins_reqd = array_map( 'trim', $plugins_deps );
				foreach( $plugins_reqd as $plugin )
					if( ! in_array( $plugin, $actv_plugins ) ) {
						$notices[] = __( 'requires other plugins to be installed and active', 'wp-clips' );
						break;
					}
			}

			// Check theme is compatible
			if( ! empty( $data['depTheme'] ) )
				if( ! in_array( $data['depTheme'], $actv_theme ) ) {
					$notices[] = __( 'is not compatible with the current theme', 'wp-clips' );
				}

			// Unclip and alert if fail
			if( count( $notices ) > 0 ) {

				$name = $data['clipName'];
				rename( $data['clipPath'], WP_CLIPS_UNCL . $data['clipFile'] );

				foreach( $notices as $notice )
					add_action( 'admin_print_footer_scripts', function() use( $name, $notice ) {
						printf( '<div class="notice notice-error"><p><b>%s</b> %s. %s.</p></div>',
								$name, $notice, __( 'Refer to Clip documentation', 'wp-clips' )
							);
					});
			}
		}
	}

	/**
	 * Rebuild Clips directory
	 * @since 3.1
	 */
	private function rebuild() {

		foreach( $this->clips() as $slug => $data ) {

			if( ! isset( $_POST[ $slug ] ) ) continue;
			extract( $data );

			switch( $_POST[ $slug ] ) {

				// Move Clip to base directory if 'active' selected
				case 'active':
					if( ! $clipActv ) {
						$path = in_array( $clipFile, [ 'clip-core', 'clip-custom' ] ) ? '' : 'precoded/';
						rename( $clipPath, WP_CLIPS_DIR . $path . $clipFile );
					}
					break;

				// Move Clip to sub directory if 'unclipped' selected
				case 'unclipped':
					if( $clipActv ) {
						rename( $clipPath, WP_CLIPS_UNCL . $clipFile );
					}
					break;

				// Check uninstalls and delete Clip if 'delete' selected
				case 'delete':
					$uninstall = $clipPath . '/uninstall.php';
					if( file_exists( $uninstall ) ) include( $uninstall );
					$this->delete( $clipPath );
			}
		}

		// Rebuild notice
		add_action( 'admin_print_footer_scripts', function() {
			printf( '<div class="notice notice-success"><p>%s.</p></div>', __( 'Clips have been rebuilt', 'wp-clips' ) );
		});
	}

	// Get Clips and data
	private function clips( $dirs = [ WP_CLIPS_DIR, WP_CLIPS_PREC, WP_CLIPS_UNCL ] ) {

		$clips = [];

		foreach( $dirs as $dir ) {

			// Get Clips and data for selected directory
			foreach( glob( $dir . 'clip-*/', GLOB_NOSORT ) as $path ) {

				$filename = basename( $path );
				$filedata = explode( '.', $filename, 2 );

				$clip = [
					'clipPath' => untrailingslashit( $path ),
					'clipFile' => $filename,
					'clipActv' => basename( dirname( $path ) ) !== 'unclipped' ? true : false
				];

				if( file_exists( $path . 'core-functions.php' ) ) {
					$clip['clipName'] = __( 'Core Clip', 'wp-clips' );
				}
				else {

					$headers = [
						'clipName' => 'Clip Name',	
						'depTheme' => 'Required Theme',
						'depsPlug' => 'Required Plugins',
					];

					if( file_exists( $path . 'custom-functions.php' ) ) {
						$clip['clipName'] = __( 'Custom Clip', 'wp-clips' );
						$file = $path . 'custom-functions.php';
					}
					else {
						if( file_exists( $path . 'clip-functions.php' ) ) {
							$file = $path . 'clip-functions.php';
						}
						elseif( file_exists( $path . 'admin-functions.php' ) ) {
							$file = $path . 'admin-functions.php';
						}
						else continue;

						$headers = $headers + [
							'clipPage' => 'Clip URI',
							'clipVers' => 'Version',
							'authName' => 'Author',
							'authPage' => 'Author URI',
							'clipData' => 'Datafile',
							'clipEdit' => 'Editable',
							'clipSafe' => 'Safe'
						];
						$clip['clipName'] = $filedata[0];
						$clip['clipVers'] = empty( $filedata[1] ) ? '' : $filedata[1];
					}
					$data = $this->get_clip_data( $file, $headers );
					$clip = array_merge( $clip, array_filter( $data ) );
				}

				$clips[ $filedata[0] ] = $clip;
			}
		}

		return $clips;
	}

	// Get file headers
	private function get_clip_data( $file, $headers ) {

		$fp = fopen( $file, 'r' );
		$data = '';

		if( $fp ) {
			$data = fread( $fp, 1 * KB_IN_BYTES );
			fclose( $fp );
		}	 

		$data = str_replace( '\r', '\n', $data );

		foreach( $headers as $field => $regex )
			if( preg_match( '/^[ \t\/*#@]*' . preg_quote( $regex, '/' ) . ':(.*)$/mi', $data, $match ) && $match[1] ) {
				$headers[ $field ] = _cleanup_header_comment( $match[1] );
			}
			else $headers[ $field ] = '';

		return $headers;
	}

	// Delete directory and contents
	private function delete( $dir ) {

		$files = array_diff( scandir( $dir ), [ '.', '..' ] );
    	foreach( $files as $file ) {
			$path = $dir . '/' . $file;
			is_dir( $path ) ? $this->delete( $path ) : unlink( $path );
		}
		return rmdir( $dir );
	}
}

new WP_Clips_Admin();